package com.example.meetup;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.InputType;
import android.view.View;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import com.budiyev.android.codescanner.AutoFocusMode;
import com.budiyev.android.codescanner.CodeScanner;
import com.budiyev.android.codescanner.CodeScannerView;
import com.budiyev.android.codescanner.DecodeCallback;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.zxing.BarcodeFormat;
import java.util.ArrayList;
import java.util.List;

public class ScannerActivity extends AppCompatActivity {
    private static final int CAMERA_REQUEST_PERMISSION = 101;
    private CodeScanner mCodeScanner;
    private FloatingActionButton switchModeFab, enterEventCodeFab, logoutFab;
    private FirebaseFirestore db = FirebaseFirestore.getInstance();
    private FirebaseAuth mAuth;
    private String currentUserCode;
    private String currentEventCode;
    private GoogleSignInClient mGoogleSignInClient;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scanner);

        mAuth = FirebaseAuth.getInstance();
        CodeScannerView scannerView = findViewById(R.id.scanner_view);
        switchModeFab = findViewById(R.id.fab_switch_mode);
        enterEventCodeFab = findViewById(R.id.fab_enter_event_code);
        enterEventCodeFab.setOnClickListener(view -> promptForEventCode());
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.CAMERA}, CAMERA_REQUEST_PERMISSION);
        } else {
            setupScanner(scannerView);
        }
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(getString(R.string.default_web_client_id))
                .requestEmail()
                .build();
        mGoogleSignInClient = GoogleSignIn.getClient(this, gso);
        switchModeFab.setOnClickListener(view -> {
            FirebaseUser currentUser = mAuth.getCurrentUser();
            if (currentUser != null) {
                db.collection("users").document(currentUser.getUid()).get()
                        .addOnSuccessListener(documentSnapshot -> {
                            if (documentSnapshot.contains("isAdmin") && documentSnapshot.getBoolean("isAdmin")) {
                                // Если пользователь уже является администратором, пропускаем ввод пароля
                                enterAdminMode();
                            } else {
                                // Иначе, показываем диалог для ввода пароля
                                showPasswordDialog();
                            }
                        })
                        .addOnFailureListener(e -> Toast.makeText(this, "Error checking admin status", Toast.LENGTH_SHORT).show());
            }
        });
        logoutFab = findViewById(R.id.fab_logout);
        logoutFab.setOnClickListener(view -> showLogoutDialog());
    }

    private void showLogoutDialog() {
        new AlertDialog.Builder(this)
                .setTitle("Logout")
                .setMessage("Are you sure you want to log out?")
                .setPositiveButton("Log out", (dialogInterface, i) -> logout())
                .setNegativeButton("Cancel", null)
                .show();
    }

    private void logout() {
        // Отключаемся от Google Sign In
        mGoogleSignInClient.signOut().addOnCompleteListener(this, task -> {
            // Firebase sign out
            FirebaseAuth.getInstance().signOut();

            // После отключения, начинаем LoginActivity
            startActivity(new Intent(ScannerActivity.this, LoginActivity.class));
            finish();
        });
    }


    private void enterAdminMode() {
        startActivity(new Intent(ScannerActivity.this, AdminPanelActivity.class));
    }

    private void showPasswordDialog() {
        final EditText passwordInput = new EditText(this);
        passwordInput.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);

        new AlertDialog.Builder(this)
                .setTitle("Admin Mode")
                .setMessage("Enter the admin password:")
                .setView(passwordInput)
                .setPositiveButton("Enter", (dialogInterface, i) -> {
                    String password = passwordInput.getText().toString();
                    if (validatePassword(password)) {
                        FirebaseUser currentUser = mAuth.getCurrentUser();
                        if (currentUser != null) {
                            db.collection("users").document(currentUser.getUid())
                                    .update("isAdmin", true)
                                    .addOnSuccessListener(aVoid -> enterAdminMode())
                                    .addOnFailureListener(e -> Toast.makeText(ScannerActivity.this, "Error setting admin status", Toast.LENGTH_SHORT).show());
                        }
                    } else {
                        Toast.makeText(ScannerActivity.this, "Incorrect password", Toast.LENGTH_SHORT).show();
                    }
                })
                .setNegativeButton("Cancel", null)
                .show();
    }

    private void setupScanner(CodeScannerView scannerView) {
        mCodeScanner = new CodeScanner(this, scannerView);
        List<BarcodeFormat> formats = new ArrayList<>();
        formats.add(BarcodeFormat.QR_CODE);
        mCodeScanner.setFormats(formats);
        mCodeScanner.setAutoFocusMode(AutoFocusMode.CONTINUOUS);
        mCodeScanner.setAutoFocusEnabled(true);
        mCodeScanner.setDecodeCallback(new DecodeCallback() {
            @Override
            public void onDecoded(@NonNull com.google.zxing.Result result) {
                runOnUiThread(() -> {
                    if (currentUserCode == null || currentEventCode == null) {
                        Toast.makeText(ScannerActivity.this, "Сначала заполните код для сканера!", Toast.LENGTH_LONG).show();
                        return;
                    }
                    // Разбиваем полученный текст на части
                    String[] parts = result.getText().split(":");
                    if (parts.length == 3) {
                        String scannedUserId = parts[0];
                        String scannedEventId = parts[1];
                        String scannedInvitationId = parts[2];

                        if (scannedUserId.equals(currentUserCode) && scannedEventId.equals(currentEventCode)) {
                            FirebaseFirestore db = FirebaseFirestore.getInstance();
                            DocumentReference invitationRef = db.collection("users").document(scannedUserId)
                                    .collection("events").document(scannedEventId)
                                    .collection("invitations").document(scannedInvitationId);

                            invitationRef.get().addOnCompleteListener(task -> {
                                if (task.isSuccessful()) {
                                    DocumentSnapshot document = task.getResult();
                                    if (document.exists() && document.getBoolean("isAccepted") != Boolean.TRUE) {
                                        // Приглашение существует и еще не было принято
                                        invitationRef.update("isAccepted", true)
                                                .addOnSuccessListener(aVoid -> {
                                                    displayResult(0);
                                                })
                                                .addOnFailureListener(e -> displayResult(3));
                                    } else if (document.getBoolean("isAccepted") == Boolean.TRUE) {
                                        // Приглашение уже было принято
                                        displayResult(2);
                                    } else {
                                        // Приглашение не существует
                                        displayResult(1);
                                    }
                                } else {
                                    // Ошибка при доступе к документу
                                    displayResult(3);
                                }
                            });
                        } else {
                            displayResult(4);
                        }
                    } else {
                        displayResult(4);
                    }
                });
            }
        });
        scannerView.setOnClickListener(view -> mCodeScanner.startPreview());
    }

    private void displayResult(int result) {
        FrameLayout colorOverlay = findViewById(R.id.color_overlay);
        TextView messageText = findViewById(R.id.message_text);
        // Определение цвета и сообщения в зависимости от результата сканирования
        if (result==0) {
            colorOverlay.setBackgroundColor(getResources().getColor(android.R.color.holo_green_dark));
            messageText.setText("ДОБРО ПОЖАЛОВАТЬ");
        } else if (result==1) {
            colorOverlay.setBackgroundColor(getResources().getColor(android.R.color.holo_red_dark));
            messageText.setText("ВАС НЕТ В СПИСКЕ");
        } else if (result==2) {
            colorOverlay.setBackgroundColor(getResources().getColor(android.R.color.holo_orange_dark));
            messageText.setText("ВЫ УЖЕ ОТМЕЧЕНЫ");
        } else if (result==3) {
            colorOverlay.setBackgroundColor(getResources().getColor(android.R.color.holo_red_dark));
            messageText.setText("ОШИБКА");
        } else if (result==4){
            colorOverlay.setBackgroundColor(getResources().getColor(android.R.color.holo_red_dark));
            messageText.setText("НЕКОРРЕКТНЫЙ QR-КОД");
        }

        colorOverlay.setVisibility(View.VISIBLE);
        messageText.setVisibility(View.VISIBLE);

        // Скрываем сообщение и цветной фон через 2 секунды
        new Handler(Looper.getMainLooper()).postDelayed(() -> {
            colorOverlay.setVisibility(View.GONE);
            messageText.setVisibility(View.GONE);
        }, 2000);
    }

    private boolean validatePassword(String password) {
        // Здесь должна быть ваша логика проверки пароля
        // Например, сравнение с хардкодированным паролем (не рекомендуется для реальных приложений!)
        return "admin".equals(password);
    }
    private void promptForEventCode() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Настройте сканнер");

        // Создание LinearLayout для вертикального расположения EditText'ов
        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);

        // Первый EditText для ввода userId
        final EditText userIdInput = new EditText(this);
        userIdInput.setInputType(InputType.TYPE_CLASS_TEXT);
        userIdInput.setHint("Код пользователя");
        layout.addView(userIdInput); // Добавление в layout

        // Второй EditText для ввода eventId
        final EditText eventIdInput = new EditText(this);
        eventIdInput.setInputType(InputType.TYPE_CLASS_TEXT);
        eventIdInput.setHint("Код мероприятия");
        layout.addView(eventIdInput); // Добавление в layout

        builder.setView(layout); // Установка layout как view диалога

        builder.setPositiveButton("OK", (dialog, which) -> {
            String userId = userIdInput.getText().toString().trim();
            String eventId = eventIdInput.getText().toString().trim();
            if (!userId.isEmpty() && !eventId.isEmpty()) {
                checkEventForUser(userId, eventId);
            } else {
                Toast.makeText(ScannerActivity.this, "Введите оба поля", Toast.LENGTH_SHORT).show();
            }
        });
        builder.setNegativeButton("Отмена", (dialog, which) -> dialog.cancel());
        builder.show();
    }

    private void checkEventForUser(String userId, String eventId) {
        FirebaseFirestore db = FirebaseFirestore.getInstance();
        db.collection("users").document(userId).collection("events").document(eventId).get().addOnCompleteListener(task -> {
            if (task.isSuccessful()) {
                DocumentSnapshot document = task.getResult();
                if (document != null && document.exists()) {
                    // Событие существует для данного пользователя
                    Toast.makeText(ScannerActivity.this, "Сканер настроен", Toast.LENGTH_SHORT).show();
                    currentUserCode = userId;
                    currentEventCode = eventId;
                } else {
                    // Событие не найдено для данного пользователя
                    Toast.makeText(ScannerActivity.this, "Нет такого мероприятия", Toast.LENGTH_SHORT).show();
                }
            } else {
                // Обработка ошибок запроса
                Toast.makeText(ScannerActivity.this, "Ошибка", Toast.LENGTH_SHORT).show();
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (mCodeScanner != null) {
            mCodeScanner.startPreview();
        }
    }

    @Override
    protected void onPause() {
        if (mCodeScanner != null) {
            mCodeScanner.releaseResources();
        }
        super.onPause();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == CAMERA_REQUEST_PERMISSION && grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            CodeScannerView scannerView = findViewById(R.id.scanner_view);
            setupScanner(scannerView);
        } else {
            Toast.makeText(this, "Camera permission is required to use QR Scanner", Toast.LENGTH_SHORT).show();
        }
    }
}
